#!/usr/bin/perl

# Script to parse the output from cnfsstat for Cacti
# Version: 1.1, 2016-10-08
# Author: Sebastian Suchanek

$CNFS_STAT = "/usr/lib/news/bin/cnfsstat";

my @data;
my @buffers;
my @buffer_data;

$mode = lc(shift(@ARGV));
if($mode =~ /^query$/){
  query_data(@ARGV);
}elsif($mode =~ /^index$/){
  print_buffers();
}elsif($mode =~ /^get$/){
  $req_stat = lc(shift(@ARGV));
  print_singledata($req_stat, @ARGV);
}elsif($mode =~ /^getall$/){
  print_alldata(@ARGV);
}else{
  print_usage();
  exit(2);
}

exit(0);


sub print_usage{
  print("
Ways to use inn-cnfsstats.pl:
inn-cnfsstats.pl index
  Retreive list of all buffers
inn-cnfsstats.pl query (buffer|size|position|cycles)
  Get specified statistic for all buffers
inn-cnfsstats.pl get (size|position|cycles) BUFFER
 Get specified statistic for buffer BUFFER
inn-cnfsstats.pl getall BUFFER
 Get all statistics for buffer BUFFER

");
}


sub get_buffers{
  @data = `$CNFS_STAT`;
  foreach $line (@data) {
    if ($line =~ m/Buffer (.+?),/) {
      push @buffers, $1;
    } 
  }
}


sub print_buffers{
  get_buffers();
  foreach $buffer (@buffers) {
    print "$buffer\n";
  }
}


sub query_data{
  my $stat = shift;
  get_buffers();
  
  foreach $buffer (@buffers) {
    getdata_for_buffer($buffer);
    if ($stat =~ /^buffer$/) {
      print "$buffer:$buffer\n";
    } else {
      print "$buffer:";
      output_single_value($stat);
      print "\n";
    }
  }
}   


sub output_single_value{
  my $stat = shift;

  if ($stat eq "size") {
    print $buffer_data[0];
  } elsif ($stat eq "position") {
    print $buffer_data[1];
  } elsif ($stat eq "cycles") {
    print $buffer_data[2];
  } else {
    exit(2);
  }
}


sub print_singledata{
  my $stat = shift;
  my $mybuffer = shift;
  getdata_for_buffer($mybuffer);
  output_single_value($stat);
} 

 
sub print_alldata{
  my $buffer = shift;
  getdata_for_buffer($buffer);
  print "size:$buffer_data[0] position:$buffer_data[1] cycles:$buffer_data[2]";
}


sub getdata_for_buffer{
  my $buffer = shift;
  @buffer_data = `$CNFS_STAT -m $buffer`;
  for my $i (0 .. $#buffer_data) {
    $buffer_data[$i] =~ s/(\s|\n)//;
  }
}

